package com.sxtvs.open.api.odata.controller;

import cn.hutool.core.util.IdUtil;
import com.sxtvs.open.api.odata.dto.wx.WxApiCreatePreAuthCodeResponse;
import com.sxtvs.open.api.odata.service.OauthService;
import com.sxtvs.open.core.auth.HttpContextUtil;
import com.sxtvs.open.core.auth.LoginRequired;
import com.sxtvs.open.core.conf.OauthConfig;
import com.sxtvs.open.core.sls.AliyunLogger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import java.util.Map;

@Controller
@RequestMapping("oauth")
public class OauthController {


    @Autowired
    private OauthService oauthService;

    @Autowired
    private AliyunLogger logger;

    @RequestMapping("douyin/code")
    @LoginRequired
    public String douyinQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/douyin/code", "state", state);
        oauthService.saveStateMapping(token, state);
        return "redirect:https://open.douyin.com/platform/oauth/connect/?" +
                "client_key=awfrca76s1petwh0&response_type=code&" +
                "scope=video.data%2Cfans.list%2Cdata.external.item%2Cdata.external.user%2Cfans.data%2Cvideo.list%2Crenew_refresh_token%2Cuser_info&" +
                "redirect_uri=https%3A%2F%2Fimport.smcic.net%2Fdouyin%2Fcallback-open&state=" + state;
    }

    @RequestMapping("douyin/callback")
    public String douyinCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/douyin/callback", "code", code, "state", state, "token", token);
        // todo 用户数据入库
        return "redirect:https://www.baidu.com";
    }

    @RequestMapping("kuaishou/code")
    @LoginRequired
    public String kuaishouQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/douyin/code", "state", state, "token", token);
        oauthService.saveStateMapping(token, state);
        return "redirect:https://open.kuaishou.com/oauth2/connect?" +
                "app_id=" + OauthConfig.kuaishouClientKey + "&response_type=code&" +
                "scope=user_info%2Cuser_video_info&" +
                "redirect_uri=https%3A%2F%2Fimport.smcic.net%2Fkuaishou%2Fcallback-open&state=" + state;
    }

    @RequestMapping("kuaishou/callback")
    public String kuaishouCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/kuaishou/callback", "code", code, "state", state, "token", token);
        // todo 用户数据入库
        return "redirect:https://www.baidu.com";
    }

    @RequestMapping("toutiao/code")
    @LoginRequired
    public String toutiaoQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/toutiao/code", "state", state, "token", token);
        return "redirect:https://open.snssdk.com/oauth/authorize/?" +
                "client_key=awfrca76s1petwh0&response_type=code&" +
                "scope=toutiao.video.data,user_info&" +
                "redirect_uri=http%3A%2F%2Fimport.smcic.net%2Ftoutiao%2Fcallback-open";
    }

    @RequestMapping("toutiao/callback")
    public String toutiaoCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/toutiao/callback", "code", code, "state", state, "token", token);
        // todo 用户数据入库
        return "redirect:https://www.baidu.com";
    }

    @RequestMapping("weibo/code")
    @LoginRequired
    public String weiboQrCode() {
        logger.info("key", "oauth/weibo/code");
        var weiboUrl = oauthService.getWeiboUrl();
        logger.info("key", "weiboQrCode", "url", weiboUrl);
        return weiboUrl;
    }

    @RequestMapping("weibo/callback")
    public String weiboCallback(String code, String state) {
        logger.info("key", "oauth/weibo/callback", "code", code);
        // todo 用户数据入库
        return "redirect:https://www.baidu.com";
    }

    @RequestMapping("bilibili/code")
    @LoginRequired
    public String bilibiliQrCode() {
        var state = IdUtil.nanoId();
        logger.info("key", "oauth/bilibili/code", "state", state);
        return "redirect:https://passport.bilibili.com/register/pc_oauth2.html#/?client_id=e79e6fdef9d446d7" +
                "&return_url=https%3A%2F%2Fimport.smcic.net%2Fbilibili%2Fcallback-open&response_type=code&state=" + state;
    }

    @RequestMapping("bilibili/callback")
    public String bilibiliCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/bilibili/callback", "code", code, "state", state, "token", token);
        // todo 用户数据入库
        return "redirect:https://www.baidu.com";
    }

    @RequestMapping("weixin/code")
    @LoginRequired
    public ModelAndView weixinQrCode() {
        logger.info("key", "oauth/weixin/code");
        WxApiCreatePreAuthCodeResponse preAuthCodeResponse = oauthService.createPreAuthCodeResponse();
        ModelAndView view = new ModelAndView();
        view.setViewName("weixin-code");
        // 这里使用预授权码跟用户token做一个映射
        view.addObject("pre_auth_code", preAuthCodeResponse.getPreAuthCode());
        return view;
    }

    @RequestMapping("weixin/{appid}/callback")
    public String callback(
            @RequestParam("auth_code") String authCode,
            @RequestParam("expires_in") Integer expiresIn,
            @PathVariable("appid") String appid) {
        logger.info("key", "oauth/{appid}/callback", "data",
                Map.of("authCode", authCode, "expiresIn", expiresIn, "appid", appid));
        // todo 用户数据入库
        return "redirect:https://www.baidu.com";
    }
}
